/**
 * @file traceStream.hpp
 * @author Pavel Brychta, http://www.xpablo.cz
 *
 * Copyright (c) 2016-24 Pavel Brychta. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
#pragma once

#include "trace.h"

class TraceStream final : public Trace {
  protected:
    Stream * mpOut = nullptr; // output stream

    static char hexascii(uint8_t n);

    void print(uint8_t severity, const char * buffer, int length);

  public:
    TraceStream() = default;

    explicit TraceStream(Stream * out)
        : mpOut(out) {
    }

    void init() override;

    void print(uint8_t severity, const __FlashStringHelper * fmt, ...) override;

    void print(uint8_t severity, const char * fmt, ...) override;

    void dump(uint8_t severity, const char * prefix, const uint8_t * address, size_t size) override;

    void dump(uint8_t severity, const __FlashStringHelper * prefix, const uint8_t * address, size_t size) override;

    void setOutput(Stream * out) {
        mpOut = out;
    }
};
